@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
     Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 
 class Bitmap : public Rectangle
   {
-  std::vector< std::vector< char > > data;	// char is faster than bool
+  std::vector< std::vector< unsigned char > > data;	// faster than bool
 
 public:
       // Creates a blank Bitmap
