@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,26 +13,26 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+
 #include "common.h"
 #include "rectangle.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 
 
 Character::Character( const Character & c ) throw()
-  : Rectangle( c ), gv( c.gv )
+  : Rectangle( c ), blobpv( c.blobpv ), gv( c.gv )
   {
-  for( unsigned int i = 0; i < c.bpv.size(); ++i )
-    shift_blockp( new Block( *c.bpv[i] ) );
+  for( unsigned int i = 0; i < blobpv.size(); ++i )
+    blobpv[i] = new Blob( *c.blobpv[i] );
   }
 
 
@@ -40,11 +41,11 @@ Character & Character::operator=( const Character & c ) throw()
   if( this != &c )
     {
     Rectangle::operator=( c );
+    for( unsigned int i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
+    blobpv = c.blobpv;
+    for( unsigned int i = 0; i < blobpv.size(); ++i )
+      blobpv[i] = new Blob( *c.blobpv[i] );
     gv = c.gv;
-    for( unsigned int i = 0; i < bpv.size(); ++i ) delete bpv[i];
-    bpv.clear();
-    for( unsigned int i = 0; i < c.bpv.size(); ++i )
-      shift_blockp( new Block( *c.bpv[i] ) );
     }
   return *this;
   }
@@ -52,39 +53,49 @@ Character & Character::operator=( const Character & c ) throw()
 
 Character::~Character() throw()
   {
-  for( unsigned int i = 0; i < bpv.size(); ++i ) delete bpv[i];
+  for( unsigned int i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
   }
 
 
-// Return the filled area of the main blocks only (no recursive)
+// Return the filled area of the main blobs only (no recursive)
 //
 int Character::area() const throw()
   {
   int a = 0;
-  for( int i = 0; i < blocks(); ++i ) a += bpv[i]->area();
+  for( int i = 0; i < blobs(); ++i ) a += blobpv[i]->area();
   return a;
   }
 
 
-Block & Character::block( int i ) throw()
+Blob & Character::blob( int i ) throw()
   {
-  if( i < 0 || i >= blocks() )
-    Ocrad::internal_error( "block, index out of bounds" );
-  return *bpv[i];
+  if( i < 0 || i >= blobs() )
+    Ocrad::internal_error( "blob, index out of bounds" );
+  return *blobpv[i];
   }
 
 
-void Character::shift_blockp( Block * p ) throw()
+Blob & Character::main_blob() throw()
+  {
+  int imax = 0;
+  for( int i = 1; i < blobs(); ++i )
+    if( blobpv[i]->size() > blobpv[imax]->size() )
+      imax = i;
+  return *blobpv[imax];
+  }
+
+
+void Character::shift_blobp( Blob * p ) throw()
   {
   add_rectangle( *p );
-  int i = blocks() - 1;
+  int i = blobs() - 1;
   for( ; i >= 0; --i )
     {
-    Block & bi = *bpv[i];
+    Blob & bi = *blobpv[i];
     if( p->vcenter() > bi.vcenter() ) break;
     if( p->vcenter() == bi.vcenter() && p->hcenter() >= bi.hcenter() ) break;
     }
-  bpv.insert( bpv.begin() + i + 1, p );
+  blobpv.insert( blobpv.begin() + i + 1, p );
   }
 
 
@@ -138,11 +149,38 @@ bool Character::maybe( int code ) const throw()
   return false;
   }
 
+/*
+bool Character::maybe_digit() const throw()
+  {
+  for( int i = 0; i < guesses(); ++i )
+    if( UCS::isdigit( gv[i].code ) ) return true;
+  return false;
+  }
+
+
+bool Character::maybe_letter() const throw()
+  {
+  for( int i = 0; i < guesses(); ++i )
+    if( UCS::isalpha( gv[i].code ) ) return true;
+  return false;
+  }
+*/
 
 void Character::join( Character & c ) throw()
   {
-  for( int i = 0; i < c.blocks(); ++i ) shift_blockp( c.bpv[i] );
-  c.bpv.clear();
+  for( int i = 0; i < c.blobs(); ++i ) shift_blobp( c.blobpv[i] );
+  c.blobpv.clear();
+  }
+
+
+unsigned char Character::byte_result() const throw()
+  {
+  if( guesses() )
+    {
+    const unsigned char ch = UCS::map_to_byte( gv[0].code );
+    if( ch ) return ch;
+    }
+  return '_';
   }
 
 
@@ -154,11 +192,12 @@ void Character::print( const Control & control ) const throw()
     switch( control.format )
       {
       case Control::byte:
-        { char ch = UCS::map_to_byte( gv[0].code );
+        { unsigned char ch = UCS::map_to_byte( gv[0].code );
         if( ch ) std::putc( ch, control.outfile ); }
         break;
       case Control::utf8:
-        std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
+        if( gv[0].code )
+          std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
       }
     }
   else std::putc( '_', control.outfile );
@@ -175,7 +214,7 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
     switch( control.format )
       {
       case Control::byte:
-        { char ch = UCS::map_to_byte( gv[i].code );
+        { unsigned char ch = UCS::map_to_byte( gv[i].code );
         if( ch ) std::fprintf( control.outfile, "guess '%c', confidence %d    ",
                                ch, gv[i].value ); }
         break;
@@ -207,9 +246,9 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
       bool iscbbot = ( row == charbox.bottom() );
 
       bool ish1top = false, ish1bot = false, ish2top = false, ish2bot = false;
-      if( blocks() == 1 && bpv[0]->holes() )
+      if( blobs() == 1 && blobpv[0]->holes() )
         {
-        const Block & b = *bpv[0];
+        const Blob & b = *blobpv[0];
         ish1top = ( row == b.hole(0).top() );
         ish1bot = ( row == b.hole(0).bottom() );
         if( b.holes() > 1 )
@@ -222,9 +261,9 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
       for( int col = left(); col <= right(); ++col )
         {
         char ch = ( isvc && col == hcenter() ) ? '+' : '';
-        for( int i = 0; i < blocks(); ++i )
+        for( int i = 0; i < blobs(); ++i )
           {
-          int id = bpv[i]->id( row, col );
+          int id = blobpv[i]->id( row, col );
           if( id != 0 )
             {
             if( id > 0 ) ch = (ch == '+') ? 'C' : 'O';
@@ -263,7 +302,7 @@ void Character::xprint( const Control & control ) const throw()
     switch( control.format )
       {
       case Control::byte:
-        { char ch = UCS::map_to_byte( gv[i].code );
+        { unsigned char ch = UCS::map_to_byte( gv[i].code );
         if( !ch ) ch = '_';
         std::fprintf( control.exportfile, ", '%c'%d", ch, gv[i].value ); }
         break;
@@ -273,3 +312,43 @@ void Character::xprint( const Control & control ) const throw()
       }
   std::fputs( "\n", control.exportfile );
   }
+
+
+void Character::apply_filter( const Filter & filter ) throw()
+  {
+  if( filter.type() == Filter::none ) return;
+  const int code = guesses() ? gv[0].code : 0;
+  bool remove = false;
+
+  switch( filter.type() )
+    {
+    case Filter::none:			// only for completeness
+      break;
+    case Filter::letters_only:
+      remove = true;
+    case Filter::letters:
+      if( !UCS::isalpha( code ) && !UCS::isspace( code ) )
+        {
+        for( int i = 1; i < guesses(); ++i )
+          if( UCS::isalpha( gv[i].code ) ) { swap_guesses( 0, i ); break; }
+        if( guesses() && !UCS::isalpha( gv[0].code ) )
+          gv[0].code = UCS::to_nearest_letter( gv[0].code );
+        if( remove && ( !guesses() || !UCS::isalpha( gv[0].code ) ) )
+          only_guess( 0, 0 );
+        }
+      break;
+    case Filter::numbers_only:
+      remove = true;
+    case Filter::numbers:
+      if( !UCS::isdigit( code ) && !UCS::isspace( code ) )
+        {
+        for( int i = 1; i < guesses(); ++i )
+          if( UCS::isdigit( gv[i].code ) ) { swap_guesses( 0, i ); break; }
+        if( guesses() && !UCS::isdigit( gv[0].code ) )
+          gv[0].code = UCS::to_nearest_digit( gv[0].code );
+        if( remove && ( !guesses() || !UCS::isdigit( gv[0].code ) ) )
+          only_guess( 0, 0 );
+        }
+      break;
+    }
+  }
