@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <algorithm>
@@ -27,29 +27,9 @@
 
 namespace {
 
-void error( const char * s ) throw() __attribute__ ((noreturn));
 void error( const char * s ) throw()
   { Ocrad::internal_error( s ); }
 
-
-int hypoti( const int c1, const int c2 )
-  {
-  long long temp = c1; temp *= temp;
-  long long target = c2; target *= target; target += temp;
-  int lower = std::max( std::abs(c1), std::abs(c2) );
-  int upper = std::abs(c1) + std::abs(c2);
-  while( upper - lower > 1 )
-    {
-    int m = ( lower + upper ) / 2;
-    temp = m; temp *= temp;
-    if( temp < target ) lower = m; else upper = m;
-    }
-  temp = lower; temp *= temp; target *= 2; target -= temp;
-  temp = upper; temp *= temp;
-  if( target < temp ) return lower;
-  else return upper;
-  }
-
 } // end namespace
 
 
@@ -57,141 +37,152 @@ Rectangle::Rectangle( const int l, const int t, const int r, const int b ) throw
   {
   if( r < l || b < t )
     {
-    std::fprintf( stderr, "l = %d, t = %d, r = %d, b = %d\n", l, t, r, b );
+    if( verbosity >= 0 )
+      std::fprintf( stderr, "l = %d, t = %d, r = %d, b = %d\n", l, t, r, b );
     error( "bad parameter building a Rectangle" );
     }
-  _left = l; _top = t; _right = r; _bottom = b;
+  left_ = l; top_ = t; right_ = r; bottom_ = b;
   }
 
 
 void Rectangle::left( const int l ) throw()
   {
-  if( l > _right ) error( "left, bad parameter resizing a Rectangle" );
-  _left = l;
+  if( l > right_ ) error( "left, bad parameter resizing a Rectangle" );
+  left_ = l;
   }
 
 
 void Rectangle::top( const int t ) throw()
   {
-  if( t > _bottom ) error( "top, bad parameter resizing a Rectangle" );
-  _top = t;
+  if( t > bottom_ ) error( "top, bad parameter resizing a Rectangle" );
+  top_ = t;
   }
 
 
 void Rectangle::right( const int r ) throw()
   {
-  if( r < _left ) error( "right, bad parameter resizing a Rectangle" );
-  _right = r;
+  if( r < left_ ) error( "right, bad parameter resizing a Rectangle" );
+  right_ = r;
   }
 
 
 void Rectangle::bottom( const int b ) throw()
   {
-  if( b < _top ) error( "bottom, bad parameter resizing a Rectangle" );
-  _bottom = b;
+  if( b < top_ ) error( "bottom, bad parameter resizing a Rectangle" );
+  bottom_ = b;
   }
 
 
 void Rectangle::height( const int h ) throw()
   {
   if( h <= 0 ) error( "height, bad parameter resizing a Rectangle" );
-  _bottom = _top + h - 1;
+  bottom_ = top_ + h - 1;
   }
 
 
 void Rectangle::width( const int w ) throw()
   {
   if( w <= 0 ) error( "width, bad parameter resizing a Rectangle" );
-  _right = _left + w - 1;
+  right_ = left_ + w - 1;
   }
 
 
 void Rectangle::add_point( const int row, const int col ) throw()
   {
-  if( row > _bottom ) _bottom = row; else if( row < _top ) _top = row;
-  if( col > _right ) _right = col;   else if( col < _left ) _left = col;
+  if( row > bottom_ ) bottom_ = row; else if( row < top_ ) top_ = row;
+  if( col > right_ ) right_ = col;   else if( col < left_ ) left_ = col;
   }
 
 
 void Rectangle::add_rectangle( const Rectangle & re ) throw()
   {
-  if( re._left < _left ) _left = re._left;
-  if( re._top < _top ) _top = re._top;
-  if( re._right > _right ) _right = re._right;
-  if( re._bottom > _bottom ) _bottom = re._bottom;
+  if( re.left_ < left_ )     left_ = re.left_;
+  if( re.top_ < top_ )       top_ = re.top_;
+  if( re.right_ > right_ )   right_ = re.right_;
+  if( re.bottom_ > bottom_ ) bottom_ = re.bottom_;
   }
 
 
 void Rectangle::enlarge( const int scale ) throw()
   {
   if( scale > 0 )
-    { _left *= scale; _top *= scale; _right *= scale; _bottom *= scale; }
+    { left_ *= scale; top_ *= scale; right_ *= scale; bottom_ *= scale; }
   }
 
 
 void Rectangle::move( const int row, const int col ) throw()
   {
-  int d = row - _top; if( d ) { _top += d; _bottom += d; }
-  d = col - _left; if( d ) { _left += d; _right += d; }
+  int d = row - top_; if( d ) { top_ += d; bottom_ += d; }
+  d = col - left_; if( d ) { left_ += d; right_ += d; }
   }
 
 
 bool Rectangle::includes( const Rectangle & re ) const throw()
   {
-  return ( _left  <= re._left  && _top    <= re._top &&
-           _right >= re._right && _bottom >= re._bottom );
+  return ( left_  <= re.left_  && top_    <= re.top_ &&
+           right_ >= re.right_ && bottom_ >= re.bottom_ );
   }
 
 
 bool Rectangle::includes( const int row, const int col ) const throw()
   {
-  return ( _left <= col && _right >= col && _top <= row && _bottom >= row );
+  return ( left_ <= col && right_ >= col && top_ <= row && bottom_ >= row );
   }
 
 
 bool Rectangle::strictly_includes( const Rectangle & re ) const throw()
   {
-  return ( _left  < re._left  && _top    < re._top &&
-           _right > re._right && _bottom > re._bottom );
+  return ( left_  < re.left_  && top_    < re.top_ &&
+           right_ > re.right_ && bottom_ > re.bottom_ );
   }
 
 
 bool Rectangle::strictly_includes( const int row, const int col ) const throw()
   {
-  return ( _left < col && _right > col && _top < row && _bottom > row );
+  return ( left_ < col && right_ > col && top_ < row && bottom_ > row );
   }
 
 
 bool Rectangle::includes_hcenter( const Rectangle & re ) const throw()
-  { return ( _left <= re.hcenter() && _right >= re.hcenter() ); }
+  { return ( left_ <= re.hcenter() && right_ >= re.hcenter() ); }
 
 
 bool Rectangle::includes_vcenter( const Rectangle & re ) const throw()
-  { return ( _top <= re.vcenter() && _bottom >= re.vcenter() ); }
+  { return ( top_ <= re.vcenter() && bottom_ >= re.vcenter() ); }
 
 
 bool Rectangle::h_includes( const Rectangle & re ) const throw()
-  { return ( _left <= re._left && _right >= re._right ); }
+  { return ( left_ <= re.left_ && right_ >= re.right_ ); }
 
 
 bool Rectangle::v_includes( const Rectangle & re ) const throw()
-  { return ( _top <= re._top && _bottom >= re._bottom ); }
+  { return ( top_ <= re.top_ && bottom_ >= re.bottom_ ); }
 
 
 bool Rectangle::h_includes( const int col ) const throw()
-  { return ( _left <= col && _right >= col ); }
+  { return ( left_ <= col && right_ >= col ); }
 
 
 bool Rectangle::v_includes( const int row ) const throw()
-  { return ( _top <= row && _bottom >= row ); }
+  { return ( top_ <= row && bottom_ >= row ); }
 
 
 bool Rectangle::h_overlaps( const Rectangle & re ) const throw()
-  { return ( _left <= re._right && _right >= re._left ); }
+  { return ( left_ <= re.right_ && right_ >= re.left_ ); }
 
 
 bool Rectangle::v_overlaps( const Rectangle & re ) const throw()
-  { return ( _top <= re._bottom && _bottom >= re._top ); }
+  { return ( top_ <= re.bottom_ && bottom_ >= re.top_ ); }
+
+
+int Rectangle::v_overlap_percent( const Rectangle & re ) const throw()
+  {
+  int ov = std::min( bottom_, re.bottom_ ) - std::max( top_, re.top_ ) + 1;
+  if( ov > 0 )
+    ov = std::max( 1, ( ov * 100 ) / std::min( height(), re.height() ) );
+  else ov = 0;
+  return ov;
+  }
 
 
 bool Rectangle::is_hcentred_in( const Rectangle & re ) const throw()
@@ -222,13 +213,23 @@ bool Rectangle::is_vcentred_in( const Rectangle & re ) const throw()
   }
 
 
+bool Rectangle::precedes( const Rectangle & re ) const throw()
+  {
+  if( right_ < re.left_ ) return true;
+  if( this->h_overlaps( re ) &&
+      ( ( top_ < re.top_ ) || ( top_ == re.top_ && left_ < re.left_ ) ) )
+    return true;
+  return false;
+  }
+
+
 bool Rectangle::h_precedes( const Rectangle & re ) const throw()
   { return ( hcenter() < re.hcenter() ); }
 
 
 bool Rectangle::v_precedes( const Rectangle & re ) const throw()
   {
-  if( _bottom < re.vcenter() || vcenter() < re._top ) return true;
+  if( bottom_ < re.vcenter() || vcenter() < re.top_ ) return true;
   if( this->includes_vcenter( re ) && re.includes_vcenter( *this ) )
     return this->h_precedes( re );
   return false;
@@ -245,28 +246,47 @@ int Rectangle::distance( const int row, const int col ) const throw()
 
 int Rectangle::h_distance( const Rectangle & re ) const throw()
   {
-  if( re._right <= _left ) return _left - re._right;
-  if( re._left >= _right ) return re._left - _right;
+  if( re.right_ <= left_ ) return left_ - re.right_;
+  if( re.left_ >= right_ ) return re.left_ - right_;
   return 0;
   }
 
 int Rectangle::h_distance( const int col ) const throw()
   {
-  if( col <= _left ) return _left - col;
-  if( col >= _right ) return col - _right;
+  if( col <= left_ ) return left_ - col;
+  if( col >= right_ ) return col - right_;
   return 0;
   }
 
 int Rectangle::v_distance( const Rectangle & re ) const throw()
   {
-  if( re._bottom <= _top ) return _top - re._bottom;
-  if( re._top >= _bottom ) return re._top - _bottom;
+  if( re.bottom_ <= top_ ) return top_ - re.bottom_;
+  if( re.top_ >= bottom_ ) return re.top_ - bottom_;
   return 0;
   }
 
 int Rectangle::v_distance( const int row ) const throw()
   {
-  if( row <= _top ) return _top - row;
-  if( row >= _bottom ) return row - _bottom;
+  if( row <= top_ ) return top_ - row;
+  if( row >= bottom_ ) return row - bottom_;
   return 0;
   }
+
+
+int Rectangle::hypoti( const int c1, const int c2 ) throw()
+  {
+  long long temp = c1; temp *= temp;
+  long long target = c2; target *= target; target += temp;
+  int lower = std::max( std::abs(c1), std::abs(c2) );
+  int upper = std::abs(c1) + std::abs(c2);
+  while( upper - lower > 1 )
+    {
+    int m = ( lower + upper ) / 2;
+    temp = m; temp *= temp;
+    if( temp < target ) lower = m; else upper = m;
+    }
+  temp = lower; temp *= temp; target *= 2; target -= temp;
+  temp = upper; temp *= temp;
+  if( target < temp ) return lower;
+  else return upper;
+  }
