@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,12 +13,12 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <cctype>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
 
 #include "common.h"
@@ -25,7 +26,7 @@
 #include "track.h"
 #include "ucs.h"
 #include "bitmap.h"
-#include "block.h"
+#include "blob.h"
 #include "character.h"
 #include "textline.h"
 
@@ -34,9 +35,9 @@
 
 namespace {
 
-int find_space( const std::vector< Character * > & cpv, int i ) throw()
+int find_space_or_hyphen( const std::vector< Character * > & cpv, unsigned int i ) throw()
   {
-  while( i < (int)cpv.size() && !cpv[i]->maybe(' ') ) ++i;
+  while( i < cpv.size() && !cpv[i]->maybe(' ') && !cpv[i]->maybe('-') ) ++i;
   return i;
   }
 
@@ -45,45 +46,54 @@ int find_space( const std::vector< Character * > & cpv, int i ) throw()
 
 void Textline::recognize2( const Charset & charset ) throw()
   {
-  if( characters() == 0 ) return;
+  if( big_initials() >= characters() ) return;
 
-  // try to recognize separately the 3 overlapped blocks of an
+  // try to recognize separately the 3 overlapped blobs of an
   // unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 3 )
+    if( !c.guesses() && c.blobs() == 3 )
       {
-      const Block & b1 = c.block( 0 );
-      const Block & b2 = c.block( 1 );
-      const Block & b3 = c.block( 2 );		// lower block
+      const Blob & b1 = c.blob( 0 );
+      const Blob & b2 = c.blob( 1 );
+      const Blob & b3 = c.blob( 2 );		// lower blob
       if( Ocrad::similar( b2.height(), b3.height(), 20 ) && !b2.h_overlaps( b3 ) &&
           b2.v_includes( b3.vcenter() ) && b3.v_includes( b2.vcenter() ) &&
           b1.bottom() < b2.top() && b1.bottom() < b3.top() )
         {
-        Character c2( new Block( b2 ) );
-        Character c3( new Block( b3 ) );
-        if( b2.h_includes( b1.hcenter() ) ) c2.shift_blockp( new Block( b1 ) );
-        else if( b3.h_includes( b1.hcenter() ) ) c3.shift_blockp( new Block( b1 ) );
-        c2.recognize1( charset, charbox( c2 ) );
-        c3.recognize1( charset, charbox( c3 ) );
-        if( c2.guesses() && c3.guesses() )
-          { c = c2; shift_characterp( new Character( c3 ) ); ++i; }
+        if( b1.height() > b2.height() && b1.height() > b3.height() )
+          {
+          Character c1( new Blob( b1 ) );
+          c1.recognize1( charset, charbox( c1 ) );
+          if( c1.guesses() ) c = c1;
+          }
+        else
+          {
+          Character c2( new Blob( b2 ) );
+          Character c3( new Blob( b3 ) );
+          if( b2.h_includes( b1.hcenter() ) ) c2.shift_blobp( new Blob( b1 ) );
+          else if( b3.h_includes( b1.hcenter() ) ) c3.shift_blobp( new Blob( b1 ) );
+          c2.recognize1( charset, charbox( c2 ) );
+          c3.recognize1( charset, charbox( c3 ) );
+          if( c2.guesses() && c3.guesses() )
+            { c = c2; shift_characterp( new Character( c3 ) ); ++i; }
+          }
         }
       }
     }
 
-  // try to recognize separately the 2 overlapped blocks of an
+  // try to recognize separately the 2 overlapped blobs of an
   // unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 2 &&
-        c.block( 0 ).v_overlaps( c.block( 1 ) ) )
+    if( !c.guesses() && c.blobs() == 2 &&
+        c.blob( 0 ).v_overlaps( c.blob( 1 ) ) )
       {
-      Character c1( new Block( c.block( 0 ) ) );
+      Character c1( new Blob( c.blob( 0 ) ) );
       c1.recognize1( charset, charbox( c1 ) );
-      Character c2( new Block( c.block( 1 ) ) );
+      Character c2( new Blob( c.blob( 1 ) ) );
       c2.recognize1( charset, charbox( c2 ) );
       if( ( c1.guesses() && c2.guesses() ) ||
           Ocrad::similar( c1.height(), c2.height(), 20 ) )
@@ -98,14 +108,28 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // remove speckles under the charbox' bottom of an unrecognized character
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 2 &&
-        c.block( 0 ).size() > 10 * c.block( 1 ).size() &&
-        c.block( 1 ).top() > charbox( c ).bottom() )
+    if( !c.guesses() && c.blobs() == 2 &&
+        c.blob( 0 ).size() > 10 * c.blob( 1 ).size() &&
+        c.blob( 1 ).top() > charbox( c ).bottom() )
       {
-      Character c1( new Block( c.block( 0 ) ) );
+      Character c1( new Blob( c.blob( 0 ) ) );
+      c1.recognize1( charset, charbox( c1 ) );
+      if( c1.guesses() ) c = c1;
+      }
+    }
+
+  // remove speckles above the charbox' top of an unrecognized character
+  for( int i = big_initials(); i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( !c.guesses() && c.blobs() == 2 &&
+        c.blob( 1 ).size() > 5 * c.blob( 0 ).size() &&
+        c.blob( 0 ).bottom() + 2 * c.blob( 0 ).height() < charbox( c ).top() )
+      {
+      Character c1( new Blob( c.blob( 1 ) ) );
       c1.recognize1( charset, charbox( c1 ) );
       if( c1.guesses() ) c = c1;
       }
@@ -113,19 +137,20 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // try to separate lightly merged characters
   // FIXME try all possible separation points
-  for( int i = 0; i < characters(); ++i )
+  // FIXME sometimes leaves unconnected blobs
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
     if( !c.guesses() && c.width() > 20 && 5 * c.width() >= 3 * c.height() &&
         5 * c.height() >= 3 * mean_height() )
       {
-      int ib = c.blocks() - 1;
+      int ib = c.blobs() - 1;
       for( int k = ib - 1; k >= 0; --k )
-        if( c.block( k ).width() > c.block( ib ).width() ) ib = k;
-      if( ib < 0 || 10 * c.block( ib ).width() < 9 * c.width() ||
-          c.block( ib ).bottom() < c.bottom() )
+        if( c.blob( k ).width() > c.blob( ib ).width() ) ib = k;
+      if( ib < 0 || 10 * c.blob( ib ).width() < 9 * c.width() ||
+          c.blob( ib ).bottom() < c.bottom() )
         continue;
-      const Block & b = c.block( ib );			// widest block
+      const Blob & b = c.blob( ib );			// widest blob
       int colmin = 0, cmin = b.height() + 1;
       for( int col = b.hpos( 30 ); col <= b.hpos( 70 ); ++col )
         {
@@ -141,18 +166,18 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( colmin <= b.left() || colmin >= b.right() ) continue;
       Rectangle r1( b.left(), b.top(), colmin - 1, b.bottom() );
       Rectangle r2( colmin + 1, b.top(), b.right(), b.bottom() );
-      Block b1( b, r1 );
+      Blob b1( b, r1 );
       b1.adjust_height(); if( 2 * b1.height() < b.height() ) continue;
-      Block b2( b, r2 );
+      Blob b2( b, r2 );
       b2.adjust_height(); if( 2 * b2.height() < b.height() ) continue;
       b1.find_holes(); b2.find_holes();
-      Character c1( new Block( b1 ) );
-      Character c2( new Block( b2 ) );
-      for( int j = 0; j < c.blocks(); ++j ) if( j != ib )
+      Character c1( new Blob( b1 ) );
+      Character c2( new Blob( b2 ) );
+      for( int j = 0; j < c.blobs(); ++j ) if( j != ib )
         {
-        const Block & bj = c.block( j );
-        if( c1.includes_hcenter( bj ) ) c1.shift_blockp( new Block( bj ) );
-        else if( c2.includes_hcenter( bj ) ) c2.shift_blockp( new Block( bj ) );
+        const Blob & bj = c.blob( j );
+        if( c1.includes_hcenter( bj ) ) c1.shift_blobp( new Blob( bj ) );
+        else if( c2.includes_hcenter( bj ) ) c2.shift_blobp( new Blob( bj ) );
         }
       c1.recognize1( charset, charbox( c1 ) );
       c2.recognize1( charset, charbox( c2 ) );
@@ -165,19 +190,19 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
     }
 
-  // try to recognize 1 block unrecognized characters with holes by
+  // try to recognize 1 blob unrecognized characters with holes by
   // removing small holes (noise)
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
-    if( !c.guesses() && c.blocks() == 1 && c.block( 0 ).holes() )
+    if( !c.guesses() && c.blobs() == 1 && c.blob( 0 ).holes() )
       {
       Character c1( c );
-      Block & b = c1.block( 0 );
+      Blob & b = c1.blob( 0 );
       for( int j = b.holes() - 1; j >= 0; --j )
         if( 64 * b.hole( j ).size() <= b.size() ||
             16 * b.hole( j ).height() <= b.height() ) b.fill_hole( j );
-      if( b.holes() < c.block( 0 ).holes() )
+      if( b.holes() < c.blob( 0 ).holes() )
         {
         c1.recognize1( charset, charbox( c1 ) );
         if( c1.guesses() ) { c = c1; continue; }
@@ -193,24 +218,24 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // separate merged characters recognized by recognize1
-  for( int i = 0; i < characters(); )		// FIXME this block sucks
+  for( int i = big_initials(); i < characters(); )		// FIXME this block sucks
     {
     if( !cpv[i]->guesses() || cpv[i]->guess( 0 ).code != 0 ) { ++i; continue; }
     Character c( *cpv[i] );
     delete_character( i );
-    if( c.guesses() >= 3 && c.blocks() >= 1 )
+    if( c.guesses() >= 3 && c.blobs() >= 1 )
       {
       int left = c.guess( 0 ).value;
       for( int g = 1; g < c.guesses(); ++g )
         {
-        Block & b = c.block( c.blocks() - 1 );
+        Blob & b = c.blob( c.blobs() - 1 );
         Rectangle re( left, b.top(), c.guess( g ).value, b.bottom() );
         b.add_rectangle( re );
-        Block b1( b, re ); b1.adjust_height(); b1.find_holes();
-        Character c1( new Block( b1 ) );
-        for( int k = 0; k < c.blocks() - 1; ++k )
-          if( !c.block( k ).includes( re ) && re.includes_hcenter( c.block( k ) ) )
-            c1.shift_blockp( new Block( c.block( k ) ) );
+        Blob b1( b, re ); b1.adjust_height(); b1.find_holes();
+        Character c1( new Blob( b1 ) );
+        for( int k = 0; k < c.blobs() - 1; ++k )
+          if( !c.blob( k ).includes( re ) && re.includes_hcenter( c.blob( k ) ) )
+            c1.shift_blobp( new Blob( c.blob( k ) ) );
         c1.add_guess( c.guess( g ).code, 0 );
         shift_characterp( new Character( c1 ) );
         left = re.right() + 1;
@@ -219,7 +244,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // choose between 'B' and 'a'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -236,8 +261,8 @@ void Textline::recognize2( const Charset & charset ) throw()
           {
           int code2 = c2.guess(0).code;
           if( code2 >= 128 ) continue;
-          if( ( std::isalpha( code2 ) && 5 * c1.height() < 4 * c2.height() ) ||
-              ( std::islower( code2 ) && !UCS::islower_small_ambiguous( code2 ) &&
+          if( ( std::isupper( code2 ) && code2 != 'B' && code2 != 'Q' && 5 * c1.height() < 4 * c2.height() ) ||
+              ( UCS::islower_small( code2 ) && code2 != 'r' && !UCS::islower_small_ambiguous( code2 ) &&
                 ( c1.height() <= c2.height() ||
                   Ocrad::similar( c1.height(), c2.height(), 10 ) ) ) )
             { c1.swap_guesses( 0, 1 ); break; }
@@ -247,7 +272,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // choose between '8' and 'a' or 'e'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -275,10 +300,19 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform some small letters to capitals
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  {
+  int begin = big_initials();
+  bool isolated = false;	// isolated letters compare with all line
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c1 = character( i );
-    if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
+    if( c1.maybe(' ') )
+      {
+      if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
+        { begin = big_initials(); isolated = true; }
+      else { begin = i + 1; isolated = false; }
+      continue;
+      }
     if( c1.guesses() == 1 )
       {
       int code = c1.guess( 0 ).code;
@@ -289,7 +323,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         {
         Character & c2 = character( j );
         if( !c2.guesses() ) continue;
-        if( c2.maybe(' ') ) break;
+        if( c2.maybe(' ') ) { if( isolated ) continue; else break; }
         int code2 = c2.guess( 0 ).code;
         if( code2 >= 128 || !std::isalpha( code2 ) ) continue;
         if( !capital )
@@ -299,11 +333,13 @@ void Textline::recognize2( const Charset & charset ) throw()
                    ( c1.height() >= c2.height() ||
                      Ocrad::similar( c1.height(), c2.height(), 10 ) ) )
             capital = true;
+          else if( code2 == 't' && c1.height() >= c2.height() ) capital = true;
+
           }
-        if( !small && std::islower( code2 ) && code2 != 'j' )
+        if( !small && std::islower( code2 ) && code2 != 'l' && code2 != 'j' )
           {
           if( 5 * c1.height() < 4 * c2.height() ) small = true;
-          else if( UCS::islower_small( code2 ) &&
+          else if( UCS::islower_small( code2 ) && code2 != 'r' &&
                    ( j < i || !UCS::islower_small_ambiguous( code2 ) ) &&
                    Ocrad::similar( c1.height(), c2.height(), 10 ) )
             small = true;
@@ -312,16 +348,17 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( capital && !small ) c1.insert_guess( 0, std::toupper( code ), 1 );
       }
     }
+  }
 
   // transform 'i' into 'j'
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'i' )
       {
       int j = i + 1;
       if( j >= characters() || !character( j ).guesses() )
-        { j = i - 1; if( j < 0 || !character( j ).guesses() ) continue; }
+        { j = i - 1; if( j < big_initials() || !character( j ).guesses() ) continue; }
       Character & c2 = character( j );
       if( UCS::isvowel( c2.guess( 0 ).code ) &&
           c1.bottom() >= c2.bottom() + ( c2.height() / 4 ) )
@@ -330,23 +367,32 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform small o or u with accent or diaeresis to capital
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  {
+  int begin = big_initials();
+  bool isolated = false;	// isolated letters compare with all line
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() >= 1 )
       {
-      if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
+      if( c1.maybe(' ') )
+        {
+        if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
+          { begin = big_initials(); isolated = true; }
+        else { begin = i + 1; isolated = false; }
+        continue;
+        }
       int code = c1.guess( 0 ).code;
-      if( code < 128 || c1.blocks() < 2 ) continue;
+      if( code < 128 || c1.blobs() < 2 ) continue;
       int codeb = UCS::base_letter( code );
       if( codeb != 'o' && codeb != 'u' ) continue;
-      const Block & b1 = c1.block( c1.blocks() - 1 );	// lower block
+      const Blob & b1 = c1.blob( c1.blobs() - 1 );	// lower blob
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
         if( c2.guesses() >= 1 )
           {
-          if( c2.maybe(' ') ) break;
+          if( c2.maybe(' ') ) { if( isolated ) continue; else break; }
           int code2 = c2.guess( 0 ).code;
           int code2b = UCS::base_letter( code2 );
           if( !code2b && code2 >= 128 ) continue;
@@ -359,9 +405,10 @@ void Textline::recognize2( const Charset & charset ) throw()
         }
       }
     }
+  }
 
   // transform 'O' or 'l' into '0' or '1'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -376,7 +423,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         if( c2.guesses() >= 1 )
           {
           int code2 = c2.guess( 0 ).code;
-          if( code2 < 128 && std::isdigit( code2 ) )
+          if( UCS::isdigit( code2 ) )
             {
             if( Ocrad::similar( c1.height(), c2.height(), 10 ) )
               c1.insert_guess( 0, (code == 'l') ? '1' : '0', c1.guess(0).value + 1 );
@@ -390,7 +437,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform a small 'p' to a capital 'P'
-  for( int i = characters() - 1; i >= 0; --i )
+  for( int i = characters() - 1; i >= big_initials(); --i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'p' )
@@ -400,7 +447,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         {
         Character & c2 = character( i + 1 );
         int code = c2.guess( 0 ).code;
-        if( code < 128 && ( std::isalnum( code ) || code == '.' || code == '|' ) )
+        if( UCS::isalnum( code ) || code == '.' || code == '|' )
           {
           valid_c2 = true;
           switch( code )
@@ -412,14 +459,14 @@ void Textline::recognize2( const Charset & charset ) throw()
             }
           }
         }
-      if( !valid_c2 && i > 0 && !character(i-1).maybe(' ') )
+      if( !valid_c2 && i > big_initials() && !character(i-1).maybe(' ') )
         cap = ( std::abs( c1.bottom() - charbox(c1).bottom() ) <= 2 );
       if( cap ) c1.only_guess( 'P', 0 );
       }
     }
 
   // transform a capital 'Y' to a small 'y'
-  for( int i = characters() - 1; i > 0; --i )
+  for( int i = characters() - 1; i > big_initials(); --i )
     {
     Character & c1 = character( i - 1 );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'Y' )
@@ -427,8 +474,7 @@ void Textline::recognize2( const Charset & charset ) throw()
       Character & c2 = character( i );
       if( !c2.guesses() ) continue;
       int code = c2.guess( 0 ).code;
-      if( UCS::isvowel( code ) || ( code < 128 && std::isalnum( code ) ) ||
-          code == '.' || code == '|' )
+      if( UCS::isalnum( code ) || code == '.' || code == '|' )
         {
         switch( code )
           {
@@ -444,12 +490,12 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // transform a SSCEDI to a CSCEDI
   if( charset.enabled( Charset::iso_8859_9 ) )
-    for( int i = 0; i < characters(); ++i )
+    for( int i = big_initials(); i < characters(); ++i )
       {
       Character & c = character( i );
       if( c.guesses() == 1 && c.guess( 0 ).code == UCS::SSCEDI )
         {
-        if( i > 0 && character( i - 1 ).guesses() )
+        if( i > big_initials() && character( i - 1 ).guesses() )
           {
           Character & c1 = character( i - 1 );
           int code = c1.guess( 0 ).code;
@@ -471,15 +517,15 @@ void Textline::recognize2( const Charset & charset ) throw()
       }
 
   // transform words like 'lO.OOO' into numbers like '10.000'
-  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
+  for( int begin = big_initials(), end = begin; begin < characters(); begin = end + 1 )
     {
-    end = find_space( cpv, begin );
+    end = find_space_or_hyphen( cpv, begin );
     if( end - begin < 2 ) continue;
     Character & c1 = character( begin );
     if( !c1.guesses() ) continue;
     int height = c1.height();
     int code = c1.guess( 0 ).code;
-    if( code == 'l' || code == 'O' || code == 'o' )
+    if( UCS::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' )
       {
       int digits = 1;
       int i = begin + 1;
@@ -489,7 +535,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         if( !c.guesses() ) break;
         bool valid = false;
         int code = c.guess(0).code;
-        if( ( code == 'l' || code == 'O' || code == 'o' ) &&
+        if( ( UCS::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' ) &&
             Ocrad::similar( c.height(), height, 10 ) )
           { valid = true; ++digits; }
         if( code == '.' || code == ',' || code == ':' || code == '+' || code == '-' )
@@ -510,9 +556,9 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // detects Roman numerals 'II', 'III' and 'IIII'
-  for( int begin = 0, end = 0; begin < characters(); begin = end + 1 )
+  for( int begin = big_initials(), end = begin; begin < characters(); begin = end + 1 )
     {
-    end = find_space( cpv, begin );
+    end = find_space_or_hyphen( cpv, begin );
     if( end - begin < 2 || end - begin > 4 ) continue;
     const int height = character( begin ).height();
     int i;
@@ -531,7 +577,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // transform a vertical bar into 'l' or 'I' (or a 'l' into an 'I')
-  for( int i = 0; i < characters(); ++i )
+  for( int i = big_initials(); i < characters(); ++i )
     {
     Character & c = character( i );
     if( c.guesses() != 1 ) continue;
@@ -539,11 +585,8 @@ void Textline::recognize2( const Charset & charset ) throw()
     if( code == '|' || code == 'l' )
       {
       int lcode = 0, rcode = 0;
-      if( i > 0 )
-        { if( character( i - 1 ).guesses() )
-            lcode = character( i - 1 ).guess( 0 ).code; }
-      else if( _big_initial && _big_initial->guesses() )
-        lcode = _big_initial->guess( 0 ).code;
+      if( i > 0 && character( i - 1 ).guesses() )
+        lcode = character( i - 1 ).guess( 0 ).code;
       if( i < characters() - 1 && character( i + 1 ).guesses() )
         rcode = character( i + 1 ).guess( 0 ).code;
       if( UCS::isupper( rcode ) &&
@@ -555,10 +598,10 @@ void Textline::recognize2( const Charset & charset ) throw()
       if( rcode == '|' && ( !lcode || !UCS::isalnum( lcode ) ) )
         {
         if( i < characters() - 2 && character( i + 2 ).guesses() &&
-            std::isalpha( character( i + 2 ).guess( 0 ).code ) )
+            UCS::isalpha( character( i + 2 ).guess( 0 ).code ) )
           { c.insert_guess( 0, 'l', 1 ); continue; }
         if( i >= 2 && character( i - 2 ).guesses() &&
-            std::isalpha( character( i - 2 ).guess( 0 ).code ) )
+            UCS::isalpha( character( i - 2 ).guess( 0 ).code ) )
           { c.insert_guess( 0, 'l', 1 ); continue; }
         }
       }
@@ -566,7 +609,7 @@ void Textline::recognize2( const Charset & charset ) throw()
 
   // transform 'l' or '|' into UCS::SINODOT
   if( charset.enabled( Charset::iso_8859_9 ) )
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -607,7 +650,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join two adjacent single quotes into a double quote
-  for( int i = 0; i < characters() - 1; ++i )
+  for( int i = big_initials(); i < characters() - 1; ++i )
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
@@ -622,7 +665,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join a comma followed by a period into a semicolon
-  for( int i = 0; i < characters() - 1; ++i )
+  for( int i = big_initials(); i < characters() - 1; ++i )
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
@@ -637,7 +680,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // choose between 'a' and 'Q'
-  for( int i = 0, begin = 0; i < characters(); ++i )
+  for( int i = big_initials(), begin = i; i < characters(); ++i )
     {
     Character & c = character( i );
     if( c.maybe(' ') ) { begin = i + 1 ; continue; }
@@ -651,7 +694,7 @@ void Textline::recognize2( const Charset & charset ) throw()
         else if( i < characters() - 1 && character( i + 1 ).guesses() )
           {
           int code1 = character( i + 1 ).guess( 0 ).code;
-          if( ( std::isdigit( code1 ) || UCS::isupper( code1 ) || code1 == 'i' ) &&
+          if( ( UCS::isdigit( code1 ) || UCS::isupper( code1 ) || code1 == 'i' ) &&
               5 * c.height() > 4 * character( i + 1 ).height() )
           c.swap_guesses( 0, 1 );
           }
@@ -672,7 +715,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join a 'n' followed by a 'I' into a 'm'
-  for( int i = 0; i < characters() - 1; ++i )
+  for( int i = big_initials(); i < characters() - 1; ++i )
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
@@ -688,7 +731,7 @@ void Textline::recognize2( const Charset & charset ) throw()
     }
 
   // join the secuence '', '/', 'o', ' ' into a '%'
-  for( int i = 0; i < characters() - 3; ++i )
+  for( int i = big_initials(); i < characters() - 3; ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == UCS::DEG )
