@@ -1,5 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,12 +24,14 @@ class Textpage : public Rectangle
   std::vector< Textblock * > tbpv;
 
 public:
-  Textpage( const Page_image & page_image,
-            const char * filename, const Control & control ) throw();
+  Textpage( const Page_image & page_image, const char * filename,
+            const Control & control, const bool layout );
   ~Textpage() throw();
 
-//  const Textblock & textblock( int i ) const throw();
+  const Textblock & textblock( int i ) const throw();
   int textblocks() const throw() { return tbpv.size(); }
+  int textlines() const throw();
+  int characters() const throw();
 
   void print( const Control & control ) const throw();
   void xprint( const Control & control ) const throw();
