@@ -1,5 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +17,7 @@
 */
 
 #include <algorithm>
+#include <cstdio>
 #include <vector>
 
 #include "common.h"
@@ -25,7 +27,6 @@
 
 namespace {
 
-void error( const char * s ) throw() __attribute__ ((noreturn));
 void error( const char * s ) throw()
   { Ocrad::internal_error( s ); }
 
@@ -139,63 +140,64 @@ Vrhomboid::Vrhomboid( const int l, const int lc, const int r, const int rc,
   {
   if( r < l || h <= 0 )
     {
-    std::fprintf( stderr, "l = %d, lc = %d, r = %d, rc = %d, h = %d\n",
-                  l, lc, r, rc, h );
+    if( verbosity >= 0 )
+      std::fprintf( stderr, "l = %d, lc = %d, r = %d, rc = %d, h = %d\n",
+                    l, lc, r, rc, h );
     error( "bad parameter building a Vrhomboid" );
     }
-  _left = l; _lvcenter = lc; _right = r; _rvcenter = rc; _height = h;
+  left_ = l; lvcenter_ = lc; right_ = r; rvcenter_ = rc; height_ = h;
   }
 
 
 void Vrhomboid::left( const int l ) throw()
   {
-  if( l > _right ) error( "left, bad parameter resizing a Vrhomboid" );
-  _left = l;
+  if( l > right_ ) error( "left, bad parameter resizing a Vrhomboid" );
+  left_ = l;
   }
 
 
 void Vrhomboid::right( const int r ) throw()
   {
-  if( r < _left ) error( "right, bad parameter resizing a Vrhomboid" );
-  _right = r;
+  if( r < left_ ) error( "right, bad parameter resizing a Vrhomboid" );
+  right_ = r;
   }
 
 
 void Vrhomboid::height( const int h ) throw()
   {
   if( h <= 0 ) error( "height, bad parameter resizing a Vrhomboid" );
-  _height = h;
+  height_ = h;
   }
 
 
 void Vrhomboid::extend_left( const int l ) throw()
   {
-  if( l > _right )
+  if( l > right_ )
     error( "extend_left, bad parameter resizing a Vrhomboid" );
-  _lvcenter = vcenter( l ); _left = l;
+  lvcenter_ = vcenter( l ); left_ = l;
   }
 
 
 void Vrhomboid::extend_right( const int r ) throw()
   {
-  if( r < _left )
+  if( r < left_ )
     error( "extend_right, bad parameter resizing a Vrhomboid" );
-  _rvcenter = vcenter( r ); _right = r;
+  rvcenter_ = vcenter( r ); right_ = r;
   }
 
 
 int Vrhomboid::vcenter( const int col ) const throw()
   {
-  const int dx = _right - _left, dy = _rvcenter - _lvcenter;
-  int vc = _lvcenter;
-  if( dx && dy ) vc += ( dy * ( col - _left ) ) / dx;
+  const int dx = right_ - left_, dy = rvcenter_ - lvcenter_;
+  int vc = lvcenter_;
+  if( dx && dy ) vc += ( dy * ( col - left_ ) ) / dx;
   return vc;
   }
 
 
 bool Vrhomboid::includes( const Rectangle & r ) const throw()
   {
-  if( r.left() < _left || r.right() > _right ) return false;
+  if( r.left() < left_ || r.right() > right_ ) return false;
   const int tl = top( r.left() ), bl = bottom( r.left() );
   const int tr = top( r.right() ), br = bottom( r.left() );
   const int t = std::max( tl, tr ), b = std::min( bl, br );
@@ -205,7 +207,7 @@ bool Vrhomboid::includes( const Rectangle & r ) const throw()
 
 bool Vrhomboid::includes( const int row, const int col ) const throw()
   {
-  if( col < _left || col > _right ) return false;
+  if( col < left_ || col > right_ ) return false;
   const int t = top( col ), b = bottom( col );
   return ( t <= row && b >= row );
   }
